from bs4 import BeautifulSoup
import urllib2
import re

"""
This script does a bit of scraping of the Ulysses Page by Page blog.

1. Crawl the index and create a list of links 
   that lead to blog pages, each corresponding to one page of Ulysses. 

2. Fix the bad links. There are a couple of known bad links on the index page.

3. Visit each link in the list. Crawl the URL and scrape for MP3 files.

4. Save lists of mp3 links to a file. 
"""


def write_set_to_file(myset,myfile):
    with open(myfile,'w') as f:
        for url in myset:
            f.write(url)
            f.write('\n')


##############################
# Step 1:
# 
# Scrape the index page to compose 
# a list of links to each page.
# 
# Each page contains one page of Ulysses.

## Only use the live page 
## when the script is working.
#url = "http://ulyssespages.blogspot.com/p/page-index.html"
#page = urllib2.urlopen(url)
#soup = BeautifulSoup( page.read(), "lxml" )

# Use a local version of the Ulysses Page by Page index.
soup = BeautifulSoup(open("page-index.html"),"lxml")

# find all links to Ulysses pages and add them to page_links
page_links = []
for a in soup.find_all('a'):
    try:
        tag = a['href']
        if 'pages' in tag:
            if re.search(r'p[0-9]',tag) is not None or re.search(r'page-',tag) is not None:
                page_links.append(a['href'])
    except:
        pass



##############################
# Step 2:
#
# Fix bad links.

with open('bad_links.txt','r') as f:
    lines = f.readlines()

# start at 1, to skip the header
good_links = []
bad_links = []
for i in range(1,len(lines)):
    contents = lines[i].split(",")
    bad_links.append(  contents[0].strip() )
    good_links.append( contents[1].strip() )




################################
# Step 3:
# 
# Crawl each individual page, 
# and look for a <source> tag
# with a attribute src=""
# 
# This contains a link to an MP3.
# 
# If the link comes from libsyn,
# it is Frank Delaney's annotated Ulysses.
# Other links come from other sources,
# and are usually other audiobook versions
# of a given section or chapter.


# for each link, find embedded media and add it to media_links
# (lots of redundancy here...)
media_links = set()
ubumexico_links = set()
other_links = set()

for url in page_links:
    
    # Start by checking if this is a bad link
    if url in bad_links:
        i = bad_links.index(url)
        url = good_links[i]


    try:
        page = urllib2.urlopen(url)
        sub_soup = BeautifulSoup(page.read(),"lxml")

        print ""
        print "------------------------"
        print url
        print "------------------------"
        print ""
        for s in sub_soup.find_all('source'):
            tagsrc = s['src']
            if 'libsyn' in tagsrc:
                media_links.add(tagsrc)
            elif 'ubumexico' in tagsrc:
                ubumexico_links.add(tagsrc)
            else:
                other_links.add(tagsrc)
            print tagsrc


    except urllib2.HTTPError:
        print ""
        print "------------------------"
        print " ### ERROR ### : %s"%(url)
        print "------------------------"
        print ""



write_set_to_file(media_links,      'OUTPUT_media_links.txt')
write_set_to_file(ubumexico_links,  'OUTPUT_ubumexico_links.txt')
write_set_to_file(other_links,      'OUTPUT_other_links.txt')


